package dep

import (
	"kpt-pasture/config"
	"kpt-pasture/module/crontab"

	"gitee.com/xuyiping_admin/pkg/logger/zaplog"
	"go.uber.org/zap"

	"go.uber.org/dig"

	"gitee.com/xuyiping_admin/pkg/cron"
	prom "github.com/prometheus/client_golang/prometheus"
)

var DataCenterCrontabCounterVec = prom.NewCounterVec(
	prom.CounterOpts{
		Namespace: "kpt-pasture",
		Subsystem: "crontab",
		Name:      "crontab",
	},
	[]string{"name"},
)

func DICrontabService() (out *cron.Crontab) {
	container := DI()
	if err := container.Provide(EntryCrontab); err != nil {
		panic(err)
	}
	if err := container.Invoke(func(c *cron.Crontab) { out = c }); err != nil {
		panic(err)
	}
	return
}

// CrontabDependency 依赖注入结构体
type CrontabDependency struct {
	dig.In

	CrontabHub crontab.Crontab // 定时任务
}

func EntryCrontab(dependency CrontabDependency) *cron.Crontab {
	cfg := config.Options()
	cs := cfg.CronSetting

	newCrontab := cron.NewCrontab(DataCenterCrontabCounterVec)
	if err := newCrontab.Bind("indicators", cs.Indicators, dependency.CrontabHub.Indicators); err != nil {
		panic(err)
	}

	if err := newCrontab.Bind("UpdateCowInfo", cs.UpdateCowInfo, dependency.CrontabHub.UpdateCowInfo); err != nil {
		panic(err)
	}

	if err := newCrontab.Bind("ImmunizationPlan", cs.ImmunizationPlan, dependency.CrontabHub.ImmunizationPlan); err != nil {
		panic(err)
	}

	if err := newCrontab.Bind("SameTimePlan", cs.SameTimePlan, dependency.CrontabHub.SameTimePlan); err != nil {
		panic(err)
	}

	if err := newCrontab.Bind("UpdateSameTime", cs.UpdateSameTime, dependency.CrontabHub.UpdateSameTime); err != nil {
		panic(err)
	}

	if err := newCrontab.Bind("SystemBasicCrontab", cs.SystemBasicCrontab, dependency.CrontabHub.SystemBasicCrontab); err != nil {
		panic(err)
	}

	if err := newCrontab.Bind("DeleteOldOriginal", cs.DeleteOldOriginal, dependency.CrontabHub.DeleteOldOriginal); err != nil {
		panic(err)
	}

	if err := newCrontab.Bind("UpdateDiseaseToCalendar", cs.UpdateDiseaseToCalendar, dependency.CrontabHub.UpdateDiseaseToCalendar); err != nil {
		zaplog.Error("EntryCrontab", zap.Any("UpdateCowEstrus", err))
		panic(err)
	}

	if err := newCrontab.Bind("UpdateCowEstrus", cs.NeckRingEstrus, dependency.CrontabHub.UpdateCowEstrus); err != nil {
		zaplog.Error("EntryCrontab", zap.Any("UpdateCowEstrus", err))
		panic(err)
	}

	if err := newCrontab.Bind("NeckRingCalculate", cs.NeckRingCalculate, dependency.CrontabHub.NeckRingCalculate); err != nil {
		zaplog.Error("EntryCrontab", zap.Any("NeckRingCalculate", err))
		panic(err)
	}

	if err := newCrontab.Bind("NeckRingMerge", cs.NeckRingMerge, dependency.CrontabHub.NeckRingOriginalMerge); err != nil {
		zaplog.Error("EntryCrontab", zap.Any("NeckRingOriginalMergeData", err))
		panic(err)
	}

	if err := newCrontab.Bind("NeckRingEstrusWarning", cs.NeckRingEstrusWarning, dependency.CrontabHub.NeckRingEstrusWarning); err != nil {
		zaplog.Error("EntryCrontab", zap.Any("NeckRingEstrusWarning", err))
		panic(err)
	}

	if err := newCrontab.Bind("NeckRingHealthWarning", cs.NeckRingHealthWarning, dependency.CrontabHub.NeckRingHealthWarning); err != nil {
		zaplog.Error("EntryCrontab", zap.Any("NeckRingHealthWarning", err))
		panic(err)
	}

	if err := newCrontab.Bind("UpdatePenBehavior", cs.UpdatePenBehavior, dependency.CrontabHub.UpdatePenBehavior); err != nil {
		zaplog.Error("EntryCrontab", zap.Any("UpdatePenBehavior", err))
		panic(err)
	}

	if err := newCrontab.Bind("UpdatePenBehaviorDaily", cs.UpdatePenBehaviorDaily, dependency.CrontabHub.UpdatePenBehaviorDaily); err != nil {
		zaplog.Error("EntryCrontab", zap.Any("UpdatePenBehaviorDaily", err))
		panic(err)
	}

	if err := newCrontab.Bind("UpdateMilkOriginal", cs.UpdateMilkOriginal, dependency.CrontabHub.UpdateMilkOriginal); err != nil {
		zaplog.Error("EntryCrontab", zap.Any("UpdateMilkOriginal", err))
		panic(err)
	}

	if err := newCrontab.Bind("UpdateActiveHabit", cs.InsertMilkDaily, dependency.CrontabHub.InsertMilkDaily); err != nil {
		zaplog.Error("EntryCrontab", zap.Any("InsertMilkDaily", err))
		panic(err)
	}

	/*err = newCrontab.Bind("GenerateWorkOrder", cs.GenerateWorkOrder, dependency.CrontabHub.GenerateAsynqWorkOrder)
	if err != nil {
		panic(err)
	}
	err = newCrontab.Bind("CowPregnant", cs.CowPregnant, dependency.CrontabHub.CowPregnant)
	if err != nil {
		panic(err)
	}*/

	return newCrontab
}
