package backend

import (
	"context"
	"fmt"
	"kpt-pasture/config"
	"kpt-pasture/util"
	"mime/multipart"
	"os"
	"path/filepath"
	"time"

	"gitee.com/xuyiping_admin/pkg/xerr"
)

func (s *StoreEntry) Photos(ctx context.Context, files []*multipart.FileHeader) ([]string, error) {
	userModel, err := s.GetUserModel(ctx)
	if err != nil {
		return nil, err
	}

	workDir := fmt.Sprintf("%s", config.WorkDir)
	pathDir := fmt.Sprintf("/files/photos/%d/%s", userModel.AppPasture.Id, time.Now().Local().Format("20060102"))
	saveDir := filepath.Join(workDir, pathDir)
	if _, err = os.Stat(saveDir); os.IsNotExist(err) {
		if err = os.MkdirAll(saveDir, 0755); err != nil {
			return nil, xerr.Customf("创建目录失败: %s", err.Error())
		}
	}

	// 处理每个文件
	filePaths := make([]string, len(files))
	for i, file := range files {
		contentType := file.Header.Get("Content-Type")
		if contentType != "image/jpeg" && contentType != "image/png" && contentType != "image/gif" {
			return nil, xerr.Customf("图片格式错误: %s", file.Filename)

		}
		if file.Size > 1024*1024*5 {
			return nil, xerr.Customf("单个图片文件不能超过5MB")

		}
		ext := filepath.Ext(file.Filename)
		if ext == "" {
			switch contentType {
			case "image/jpeg":
				ext = ".jpg"
			case "image/png":
				ext = ".png"
			case "image/gif":
				ext = ".gif"
			default:
				ext = ".jpg" // 默认
			}
		}
		randomName := util.GenerateRandomNumberString(32)
		finalFilename := randomName + ext
		fPath := filepath.Join(saveDir, finalFilename)
		urlPath := filepath.Join(pathDir, finalFilename)

		if err = util.SaveUploadedFile(file, fPath); err != nil {
			return nil, xerr.Customf("保存文件失败: %s", err.Error())
		}
		filePaths[i] = urlPath
	}
	return filePaths, nil
}
